package com.glxp.api.admin.controller.inventory;

import com.github.pagehelper.PageInfo;
import com.glxp.api.admin.entity.inventory.InvProductDetailEntity;
import com.glxp.api.admin.entity.inventory.InvProductEntity;
import com.glxp.api.admin.req.inout.DeleteRequest;
import com.glxp.api.admin.req.inventory.FilterInvProductDetailRequest;
import com.glxp.api.admin.req.inventory.FilterInvProductRequest;
import com.glxp.api.admin.res.PageSimpleResponse;
import com.glxp.api.admin.res.inventory.InvProductResponse;
import com.glxp.api.admin.service.inventory.InvProductDetailService;
import com.glxp.api.admin.service.inventory.InvProductService;
import com.glxp.api.common.enums.ResultEnum;
import com.glxp.api.common.res.BaseResponse;
import com.glxp.api.common.util.ResultVOUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

@RestController
public class InvProductsController {

    @Resource
    InvProductService invProductService;
    @Resource
    InvProductDetailService invProductDetailService;

    @GetMapping("spms/inv/products/filter")
    public BaseResponse filterInvProduct(FilterInvProductRequest filterInvProductRequest) {
        List<InvProductResponse> invProductResponses = invProductService.filterJoinInvProduct(filterInvProductRequest);
        PageInfo<InvProductResponse> pageInfo;
        pageInfo = new PageInfo<>(invProductResponses);
        PageSimpleResponse<InvProductResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(invProductResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @GetMapping("spms/inv/products/filterDetail")
    public BaseResponse filterInvProductDetail(FilterInvProductDetailRequest filterInvProductRequest) {
        List<InvProductDetailEntity> invProductDetailEntities = invProductDetailService.filterInvProduct(filterInvProductRequest);
        PageInfo<InvProductDetailEntity> pageInfo;
        pageInfo = new PageInfo<>(invProductDetailEntities);
        PageSimpleResponse<InvProductDetailEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(invProductDetailEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @PostMapping("spms/inv/products/delete")
    public BaseResponse deleteInvProduct(@RequestBody DeleteRequest deleteRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        String id = deleteRequest.getId();
        InvProductEntity invProductEntity = invProductService.selectById(id);
        if (invProductEntity != null) {
            invProductService.deleteById(id);
            invProductDetailService.deleteByProductId(invProductEntity.getId() + "");
            return ResultVOUtils.success("删除成功");
        } else {
            return ResultVOUtils.error(500, "删除失败！");
        }


    }
}
