package com.glxp.api.admin.service.inventory.impl;

import com.github.pagehelper.PageHelper;
import com.glxp.api.admin.dao.inventory.InvProductDao;
import com.glxp.api.admin.entity.inventory.InvProductEntity;
import com.glxp.api.admin.req.inventory.FilterInvProductRequest;
import com.glxp.api.admin.res.inventory.InvProductResponse;
import com.glxp.api.admin.service.inventory.InvProductService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

@Service
public class InvProductServiceImpl implements InvProductService {

    @Resource
    InvProductDao invProductDao;

    @Override
    public List<InvProductEntity> filterInvProduct(FilterInvProductRequest filterInvProductRequest) {
        if (filterInvProductRequest == null) {
            return Collections.emptyList();
        }
        if (filterInvProductRequest.getPage() != null) {
            int offset = (filterInvProductRequest.getPage() - 1) * filterInvProductRequest.getLimit();
            PageHelper.offsetPage(offset, filterInvProductRequest.getLimit());
        }
        List<InvProductEntity> data = invProductDao.filterInvProduct(filterInvProductRequest);
        return data;
    }

    @Override
    public List<InvProductResponse> filterJoinInvProduct(FilterInvProductRequest filterInvProductRequest) {
        if (filterInvProductRequest == null) {
            return Collections.emptyList();
        }
        if (filterInvProductRequest.getPage() != null) {
            int offset = (filterInvProductRequest.getPage() - 1) * filterInvProductRequest.getLimit();
            PageHelper.offsetPage(offset, filterInvProductRequest.getLimit());
        }
        List<InvProductResponse> data = invProductDao.filterJoinInvProduct(filterInvProductRequest);
        return data;
    }

    @Override
    public InvProductEntity selectByUuid(String uuid, String batchNo) {
        FilterInvProductRequest filterInvProductRequest = new FilterInvProductRequest();
        filterInvProductRequest.setRelIdFk(uuid);
        filterInvProductRequest.setBatchNo(batchNo);
        List<InvProductEntity> data = filterInvProduct(filterInvProductRequest);
        if (data != null && data.size() > 0) {
            return data.get(0);
        }
        return null;
    }

    @Override
    public InvProductEntity selectById(String id) {
        return invProductDao.selectById(id);
    }

    @Override
    public boolean insertInvProduct(InvProductEntity invProductEntity) {
        return invProductDao.insertInvProduct(invProductEntity);
    }

    @Override
    public boolean updateInvProduct(InvProductEntity invProductEntity) {
        return invProductDao.updateInvProduct(invProductEntity);
    }

    @Override
    public boolean deleteById(String id) {
        return invProductDao.deleteById(id);
    }
}
